#!/bin/bash
#set -x
#-- -----------------------------------------------------------------------------
#-- Script : /home/oracle/scripts/stb_scripts/manual_standby.sh
#-- Task   : Aktualisierung der Standby Datenbank
#-- Author : Stefan M. Dohn, 19.04.2021
#-- 19.04.2021  SDohn  Creation from scrap
#-- Last updates:
#-- -----------------------------------------------------------------------------
export ME=$(whoami)
export SCRIPT_PATH=$(dirname $(readlink -f $0))
export SCRIPT_NAME=$(basename $0 .sh)
export DLOG="manual_standby_$(date +%d%m%Y).log"
export PRIMARY_DB="tl1"
export PRIMARY_SERVER="techlab1"
export STANDBY_DB="tl2"
export STANDBY_SERVER="techlab2"
MAILTO="oracle"
WARNTHRESHOLD=10

if [ "x$ME" != "xoracle" ]; then echo "Script muss als User oracle laufen" && exit 1; fi

#ENV to set for techlab1
if  [ $HOSTNAME = "techlab1" ] && [ $USER = "oracle" ]
        then
        ORACLE_SID=TL1; export ORACLE_SID
        ORACLE_HOME=/oracle/product/19.0.0/dbhome_1; export ORACLE_HOME
        echo "Skript soll nur auf Standby DB laufen! ... exit" | tee -a $DLOG
	exit 0 #Skript soll nur auf der Standby DB laufen

#ENV to set for techlab2
elif [ $HOSTNAME = "techlab2" ] && [ $USER = "oracle" ]
        then
        ORACLE_SID=TL2; export ORACLE_SID
        ORACLE_HOME=/oracle/product/19.0.0/dbhome_1; export ORACLE_HOME
fi


#===  FUNCTION  ================================================================
#         NAME:  PRMARC
#  DESCRIPTION:  Archivierung der aktuellen Log-Dateien der Primaerdatenbank TL1
#      CREATED:  19.04.2021 13:00 CET / AUTHOR:  Stefan M. Dohn
#===============================================================================
#
PRMARC()
{
# Archivierung der aktuellen Log-Dateien der Primaerdatenbank TL1
$ORACLE_HOME/bin/sqlplus sys/ora-adm@$PRIMARY_DB as sysdba <<'EOF' | tee -a $DLOG
alter system archive log current;
exit
EOF
}


#===  FUNCTION  ================================================================
#         NAME:  PRMTRST
#  DESCRIPTION:  Archivelog Transport zur Standby-Seite
#  DESCRIPTION:  Vom Host techlab1 zum lokalen Archivelog Verzeichnis
#      CREATED:  19.04.2021 13:00 CET / AUTHOR:  Stefan M. Dohn
#===============================================================================
#
PRMTRST()
{
rsync -e ssh -Pazv techlab1:/oracle/oralog/fast_recovery_area/TL1/archivelog/* /oracle/oralog/fast_recovery_area/TL2/archivelog | tee -a $DLOG

# Recovery der Standby DB
$ORACLE_HOME/bin/sqlplus sys/ora-adm@$STANDBY_DB as sysdba <<'EOF' | tee -a $DLOG
recover automatic standby database;
cancel
exit
EOF
}


#===  FUNCTION  ================================================================
#         NAME:  ARCHLAG
#  DESCRIPTION:  Den LAG der Archive Log SEQ Nummern zwischen PRIMARY und STANDBY ermittteln
#      CREATED:  19.04.2021 13:00 CET / AUTHOR:  Stefan M. Dohn
#===============================================================================
#
ARCHLAG()
{
echo "########## Prüfung auf GAP der Archive-Log-Sequence-Nummern     ##########" | tee -a $DLOG
echo "########## zwischen Primaer-DB und Standby-DB : `date '+%D %T'` ##########" | tee -a $DLOG
PRIMARY=` ssh $PRIMARY_SERVER 'source /home/oracle/scripts/setEnv.sh; sqlplus -silent / as sysdba <<EOF
set pagesize 0 feedback off verify off heading off echo off
select max(sequence#) from sys.v\\$log_history;
exit;
EOF' `

STANDBY=`source /home/oracle/scripts/setEnv.sh;sqlplus -silent / as sysdba <<.eof.
set pages 0 feed off heading off echo off verify off termout off
select max(sequence#) from sys.v\\$log_history;
exit
.eof.`

echo "Primaer-DB "$PRIMARY_DB"-"$PRIMARY | tee -a $DLOG
echo "Standby-DB "$STANDBY_DB"-"$STANDBY | tee -a $DLOG
LACK=`expr $PRIMARY - $STANDBY`
if [ $LACK -gt $WARNTHRESHOLD ]
then
echo "!!! Standby-DB ist veraltet !!!" | tee -a $DLOG
echo "########## Warn Mail versenden: `date '+%D %T'` ##########" | tee -a $DLOG
TMP=/tmp/tmp$PID
TMPM=$TMP.mail
echo "WARNUNG Standby-Datenbank ist scheinbar OUTDATED" >$TMPM
echo "Primary-DB "$PRIMARY_DB"-"$PRIMARY >> $TMPM
echo "Standby-DB "$STANDBY_DB"-"$STANDBY >> $TMPM
echo "Standby liegt $LACK Archive-Log-Dateien zurueck" >> $TMPM
echo "Die Situation ist erst kritisch wenn diese Nachricht mehrmals hintereinander kommt." >> $TMPM
echo "ACHTUNG "$SCRIPT_PATH"/archive_remove.block wird gesetzt zum Blocken der Bereinigung" >> $TMPM
mail < $TMPM -s "WARNUNG!: Standby-DB $STANDBY_DB nicht aktuell" "$MAILTO"
rm $TMPM
# Verhindern das Archive-Logs geloescht werden (separates Skript)
echo "NICHT LOESCHEN" > $SCRIPT_PATH/archive_remove.block
fi
}


#===  FUNCTION  ================================================================
#         NAME:  ARCHAGE
#  DESCRIPTION:  Alte Archivelogs auf Standby loeschen aelter als 8 Tage
#      CREATED:  19.04.2021 13:00 CET / AUTHOR:  Stefan M. Dohn
#===============================================================================
#
ARCHAGE()
{
if [ -e "$SCRIPT_PATH"/archive_remove.block ]

then
echo "!!! Archivelogs duerfen nicht geloescht werden !!!" | tee -a $DLOG
echo "Eventuell ist manuelle Intervention auf d. StandBy DB nötig: `date '+%D %T'` ##########" | tee -a $DLOG
echo "Wenn alles OK, kann das lock file $SCRIPT_PATH/archive_remove.block gelöscht werden" | tee -a $DLOG
exit;
fi

# Alte Archivelogs auf Standby loeschen aelter als 8 Tage
find /oracle/oralog/fast_recovery_area/TL2/archivelog -type f -mtime +8 -exec rm {} \; | tee -a $DLOG

# Alte Archivelogs auf Primary loeschen aelter als 8 Tage
# Vorraussetzung: Taegliches RMAN Backup der Archivelog Dateien
rman target sys/ora-adm@tl1 <<'EOF' | tee -a $DLOG
DELETE NOPROMPT ARCHIVELOG ALL COMPLETED BEFORE 'SYSDATE-8';
crosscheck archivelog all;
exit
EOF
}


PRMARC   #Archivierung der aktuellen Log-Dateien der Primaerdatenbank TL1
PRMTRST  #Archivelog Transport zur Standby-Seite
ARCHLAG  #Den LAG der Archive Log SEQ Nummern zwischen PRIMARY und STANDBY ermittteln
ARCHAGE  #Alte Archivelogs auf Standby loeschen aelter als 8 Tage

exit

